/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a activitiesadaptor -i Activities.h -l Activities /usr/src/ports/kactivities/kactivities-4.13.3-2.x86_64/src/kactivities-4.13.3/src/common/dbus/org.kde.ActivityManager.Activities.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "activitiesadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ActivitiesAdaptor
 */

ActivitiesAdaptor::ActivitiesAdaptor(Activities *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ActivitiesAdaptor::~ActivitiesAdaptor()
{
    // destructor
}

QString ActivitiesAdaptor::ActivityIcon(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.ActivityIcon
    return parent()->ActivityIcon(activity);
}

ActivityInfo ActivitiesAdaptor::ActivityInformation(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.ActivityInformation
    return parent()->ActivityInformation(activity);
}

QString ActivitiesAdaptor::ActivityName(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.ActivityName
    return parent()->ActivityName(activity);
}

int ActivitiesAdaptor::ActivityState(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.ActivityState
    return parent()->ActivityState(activity);
}

QString ActivitiesAdaptor::AddActivity(const QString &name)
{
    // handle method call org.kde.ActivityManager.Activities.AddActivity
    return parent()->AddActivity(name);
}

QString ActivitiesAdaptor::CurrentActivity()
{
    // handle method call org.kde.ActivityManager.Activities.CurrentActivity
    return parent()->CurrentActivity();
}

QStringList ActivitiesAdaptor::ListActivities(int state)
{
    // handle method call org.kde.ActivityManager.Activities.ListActivities
    return parent()->ListActivities(state);
}

QStringList ActivitiesAdaptor::ListActivities()
{
    // handle method call org.kde.ActivityManager.Activities.ListActivities
    return parent()->ListActivities();
}

ActivityInfoList ActivitiesAdaptor::ListActivitiesWithInformation()
{
    // handle method call org.kde.ActivityManager.Activities.ListActivitiesWithInformation
    return parent()->ListActivitiesWithInformation();
}

void ActivitiesAdaptor::RemoveActivity(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.RemoveActivity
    parent()->RemoveActivity(activity);
}

void ActivitiesAdaptor::SetActivityIcon(const QString &activity, const QString &icon)
{
    // handle method call org.kde.ActivityManager.Activities.SetActivityIcon
    parent()->SetActivityIcon(activity, icon);
}

void ActivitiesAdaptor::SetActivityName(const QString &activity, const QString &name)
{
    // handle method call org.kde.ActivityManager.Activities.SetActivityName
    parent()->SetActivityName(activity, name);
}

bool ActivitiesAdaptor::SetCurrentActivity(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.SetCurrentActivity
    return parent()->SetCurrentActivity(activity);
}

void ActivitiesAdaptor::StartActivity(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.StartActivity
    parent()->StartActivity(activity);
}

void ActivitiesAdaptor::StopActivity(const QString &activity)
{
    // handle method call org.kde.ActivityManager.Activities.StopActivity
    parent()->StopActivity(activity);
}


#include "activitiesadaptor.moc"
