/****************************************************************************
** Meta object code from reading C++ file 'resourceinstance.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/kactivities-4.13.3/src/lib/core/resourceinstance.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'resourceinstance.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_KActivities__ResourceInstance[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       5,   49, // properties
       1,   64, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: signature, parameters, type, tag, flags
      31,   30,   30,   30, 0x05,

 // slots: signature, parameters, type, tag, flags
      47,   30,   30,   30, 0x0a,
      64,   30,   30,   30, 0x0a,
      82,   30,   30,   30, 0x0a,
     108,  101,   30,   30, 0x0a,
     130,  121,   30,   30, 0x0a,
     157,  151,   30,   30, 0x0a,

 // properties: name, type, flags
     180,  175, 0x11095103,
     121,  184, 0x0a095103,
     151,  184, 0x0a095103,
     196,  192, 0x00095009,
     215,  202, 0x00095009,

 // enums: name, flags, count, data
     202, 0x0,    5,   68,

 // enum data: key, value
     228, uint(KActivities::ResourceInstance::User),
     233, uint(KActivities::ResourceInstance::Scheduled),
     243, uint(KActivities::ResourceInstance::Heuristic),
     253, uint(KActivities::ResourceInstance::System),
     260, uint(KActivities::ResourceInstance::World),

       0        // eod
};

static const char qt_meta_stringdata_KActivities__ResourceInstance[] = {
    "KActivities::ResourceInstance\0\0"
    "requestsFocus()\0notifyModified()\0"
    "notifyFocusedIn()\0notifyFocusedOut()\0"
    "newUri\0setUri(QUrl)\0mimetype\0"
    "setMimetype(QString)\0title\0setTitle(QString)\0"
    "QUrl\0uri\0QString\0WId\0winId\0AccessReason\0"
    "accessReason\0User\0Scheduled\0Heuristic\0"
    "System\0World\0"
};

void KActivities::ResourceInstance::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        ResourceInstance *_t = static_cast<ResourceInstance *>(_o);
        switch (_id) {
        case 0: _t->requestsFocus(); break;
        case 1: _t->notifyModified(); break;
        case 2: _t->notifyFocusedIn(); break;
        case 3: _t->notifyFocusedOut(); break;
        case 4: _t->setUri((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 5: _t->setMimetype((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->setTitle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData KActivities::ResourceInstance::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject KActivities::ResourceInstance::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_KActivities__ResourceInstance,
      qt_meta_data_KActivities__ResourceInstance, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &KActivities::ResourceInstance::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *KActivities::ResourceInstance::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *KActivities::ResourceInstance::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_KActivities__ResourceInstance))
        return static_cast<void*>(const_cast< ResourceInstance*>(this));
    return QObject::qt_metacast(_clname);
}

int KActivities::ResourceInstance::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
      else if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = uri(); break;
        case 1: *reinterpret_cast< QString*>(_v) = mimetype(); break;
        case 2: *reinterpret_cast< QString*>(_v) = title(); break;
        case 3: *reinterpret_cast< WId*>(_v) = winId(); break;
        case 4: *reinterpret_cast< AccessReason*>(_v) = accessReason(); break;
        }
        _id -= 5;
    } else if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: setUri(*reinterpret_cast< QUrl*>(_v)); break;
        case 1: setMimetype(*reinterpret_cast< QString*>(_v)); break;
        case 2: setTitle(*reinterpret_cast< QString*>(_v)); break;
        }
        _id -= 5;
    } else if (_c == QMetaObject::ResetProperty) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void KActivities::ResourceInstance::requestsFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 0, 0);
}
QT_END_MOC_NAMESPACE
