/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a activityrankingadaptor -i ActivityRanking.h -l ActivityRanking /usr/src/ports/kactivities/kactivities-4.13.3-2.x86_64/src/kactivities-4.13.3/src/service/plugins/activityranking/org.kde.ActivityManager.ActivityRanking.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "activityrankingadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ActivityRankingAdaptor
 */

ActivityRankingAdaptor::ActivityRankingAdaptor(ActivityRanking *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ActivityRankingAdaptor::~ActivityRankingAdaptor()
{
    // destructor
}

ActivityDataList ActivityRankingAdaptor::activities()
{
    // handle method call org.kde.ActivityManager.ActivityRanking.activities
    return parent()->activities();
}

QStringList ActivityRankingAdaptor::topActivities()
{
    // handle method call org.kde.ActivityManager.ActivityRanking.topActivities
    return parent()->topActivities();
}


#include "activityrankingadaptor.moc"
