/****************************************************************************
** Meta object code from reading C++ file 'Call.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kactivities-4.13.3/src/service/jobs/general/Call.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Call.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Jobs__General__Call[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       4,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
      29,   20, 0x88095103,
      46,   38, 0x0a095103,
      51,   38, 0x0a095103,
      65,   60, 0x01095103,

       0        // eod
};

static const char qt_meta_stringdata_Jobs__General__Call[] = {
    "Jobs::General::Call\0QObject*\0receiver\0"
    "QString\0slot\0argument\0bool\0waitFinished\0"
};

void Jobs::General::Call::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObjectExtraData Jobs::General::Call::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Jobs::General::Call::staticMetaObject = {
    { &Job::staticMetaObject, qt_meta_stringdata_Jobs__General__Call,
      qt_meta_data_Jobs__General__Call, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Jobs::General::Call::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Jobs::General::Call::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Jobs::General::Call::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Jobs__General__Call))
        return static_cast<void*>(const_cast< Call*>(this));
    return Job::qt_metacast(_clname);
}

int Jobs::General::Call::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Job::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
     if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QObject**>(_v) = receiver(); break;
        case 1: *reinterpret_cast< QString*>(_v) = slot(); break;
        case 2: *reinterpret_cast< QString*>(_v) = argument(); break;
        case 3: *reinterpret_cast< bool*>(_v) = waitFinished(); break;
        }
        _id -= 4;
    } else if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: setReceiver(*reinterpret_cast< QObject**>(_v)); break;
        case 1: setSlot(*reinterpret_cast< QString*>(_v)); break;
        case 2: setArgument(*reinterpret_cast< QString*>(_v)); break;
        case 3: setWaitFinished(*reinterpret_cast< bool*>(_v)); break;
        }
        _id -= 4;
    } else if (_c == QMetaObject::ResetProperty) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_END_MOC_NAMESPACE
