#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'MainConfigurationWidgetBase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.6
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINCONFIGURATIONWIDGETBASE_H
#define UI_MAINCONFIGURATIONWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QGraphicsView>
#include <QtGui/QGroupBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QRadioButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QTabWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "knuminput.h"

QT_BEGIN_NAMESPACE

class Ui_MainConfigurationWidgetBase
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *checkEnableActivities;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelRememberDocuments;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *radioRememberAllApplications;
    QRadioButton *radioDontRememberApplications;
    QRadioButton *radioRememberSpecificApplications;
    QSpacerItem *horizontalSpacer_2;
    QGroupBox *groupHistory;
    QHBoxLayout *horizontalLayout_7;
    QLabel *labelKeepHistory;
    KIntSpinBox *spinKeepHistory;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonClearRecentHistory;
    QGroupBox *groupApplications;
    QVBoxLayout *verticalLayout_5;
    QGraphicsView *viewBlacklistedApplications;
    QCheckBox *checkBlacklistAllNotOnList;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QWidget *MainConfigurationWidgetBase)
    {
        if (MainConfigurationWidgetBase->objectName().isEmpty())
            MainConfigurationWidgetBase->setObjectName(QString::fromUtf8("MainConfigurationWidgetBase"));
        MainConfigurationWidgetBase->resize(760, 613);
        verticalLayout = new QVBoxLayout(MainConfigurationWidgetBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        checkEnableActivities = new QCheckBox(MainConfigurationWidgetBase);
        checkEnableActivities->setObjectName(QString::fromUtf8("checkEnableActivities"));

        verticalLayout->addWidget(checkEnableActivities);

        tabWidget = new QTabWidget(MainConfigurationWidgetBase);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        labelRememberDocuments = new QLabel(tab);
        labelRememberDocuments->setObjectName(QString::fromUtf8("labelRememberDocuments"));

        verticalLayout_2->addWidget(labelRememberDocuments);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        radioRememberAllApplications = new QRadioButton(tab);
        radioRememberAllApplications->setObjectName(QString::fromUtf8("radioRememberAllApplications"));
        radioRememberAllApplications->setChecked(true);

        verticalLayout_3->addWidget(radioRememberAllApplications);

        radioDontRememberApplications = new QRadioButton(tab);
        radioDontRememberApplications->setObjectName(QString::fromUtf8("radioDontRememberApplications"));

        verticalLayout_3->addWidget(radioDontRememberApplications);

        radioRememberSpecificApplications = new QRadioButton(tab);
        radioRememberSpecificApplications->setObjectName(QString::fromUtf8("radioRememberSpecificApplications"));

        verticalLayout_3->addWidget(radioRememberSpecificApplications);


        horizontalLayout->addLayout(verticalLayout_3);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        groupHistory = new QGroupBox(tab);
        groupHistory->setObjectName(QString::fromUtf8("groupHistory"));
        groupHistory->setFlat(true);
        horizontalLayout_7 = new QHBoxLayout(groupHistory);
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        labelKeepHistory = new QLabel(groupHistory);
        labelKeepHistory->setObjectName(QString::fromUtf8("labelKeepHistory"));

        horizontalLayout_7->addWidget(labelKeepHistory);

        spinKeepHistory = new KIntSpinBox(groupHistory);
        spinKeepHistory->setObjectName(QString::fromUtf8("spinKeepHistory"));

        horizontalLayout_7->addWidget(spinKeepHistory);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);

        buttonClearRecentHistory = new QPushButton(groupHistory);
        buttonClearRecentHistory->setObjectName(QString::fromUtf8("buttonClearRecentHistory"));

        horizontalLayout_7->addWidget(buttonClearRecentHistory);


        horizontalLayout->addWidget(groupHistory);


        verticalLayout_2->addLayout(horizontalLayout);

        groupApplications = new QGroupBox(tab);
        groupApplications->setObjectName(QString::fromUtf8("groupApplications"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(groupApplications->sizePolicy().hasHeightForWidth());
        groupApplications->setSizePolicy(sizePolicy);
        groupApplications->setFlat(true);
        verticalLayout_5 = new QVBoxLayout(groupApplications);
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        viewBlacklistedApplications = new QGraphicsView(groupApplications);
        viewBlacklistedApplications->setObjectName(QString::fromUtf8("viewBlacklistedApplications"));
        viewBlacklistedApplications->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

        verticalLayout_5->addWidget(viewBlacklistedApplications);

        checkBlacklistAllNotOnList = new QCheckBox(groupApplications);
        checkBlacklistAllNotOnList->setObjectName(QString::fromUtf8("checkBlacklistAllNotOnList"));

        verticalLayout_5->addWidget(checkBlacklistAllNotOnList);


        verticalLayout_2->addWidget(groupApplications);

        verticalSpacer_2 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        tabWidget->addTab(tab, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(MainConfigurationWidgetBase);
        QObject::connect(checkEnableActivities, SIGNAL(toggled(bool)), tabWidget, SLOT(setEnabled(bool)));
        QObject::connect(radioDontRememberApplications, SIGNAL(toggled(bool)), groupHistory, SLOT(setHidden(bool)));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MainConfigurationWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *MainConfigurationWidgetBase)
    {
        checkEnableActivities->setText(tr2i18n("Enable activities management", 0));
        labelRememberDocuments->setText(tr2i18n("Remember opened documents:", 0));
        radioRememberAllApplications->setText(tr2i18n("For all applications", 0));
        radioDontRememberApplications->setText(tr2i18n("Do not remember", 0));
        radioRememberSpecificApplications->setText(tr2i18n("Only for specific applications", 0));
        labelKeepHistory->setText(tr2i18n("Keep history", 0));
        buttonClearRecentHistory->setText(tr2i18n("Clear recent history", 0));
        groupApplications->setTitle(QString());
        checkBlacklistAllNotOnList->setText(tr2i18n("Blacklist all applications not on this list", 0));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("Privacy", 0));
        Q_UNUSED(MainConfigurationWidgetBase);
    } // retranslateUi

};

namespace Ui {
    class MainConfigurationWidgetBase: public Ui_MainConfigurationWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // MAINCONFIGURATIONWIDGETBASE_H

