/****************************************************************************
** Meta object code from reading C++ file 'Resources.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kactivities-4.13.3/src/service/Resources.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Resources.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Resources[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
      44,   10,

 // signals: signature, parameters, type, tag, flags
      67,   61,   60,   60, 0x05,
     105,   98,   60,   60, 0x05,
     153,  140,   60,   60, 0x05,
     207,  197,   60,   60, 0x05,

 // slots: signature, parameters, type, tag, flags
     286,  248,   60,   60, 0x0a,
     340,  140,   60,   60, 0x0a,
     382,  197,   60,   60, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_Resources[] = {
    "Resources\0org.kde.ActivityManager.Resources\0"
    "D-Bus Interface\0\0event\0"
    "RegisteredResourceEvent(Event)\0events\0"
    "ProcessedResourceEvents(EventList)\0"
    "uri,mimetype\0RegisteredResourceMimeType(QString,QString)\0"
    "uri,title\0RegisteredResourceTitle(QString,QString)\0"
    "application,windowId,uri,event,reason\0"
    "RegisterResourceEvent(QString,uint,QString,uint,uint)\0"
    "RegisterResourceMimeType(QString,QString)\0"
    "RegisterResourceTitle(QString,QString)\0"
};

void Resources::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        Resources *_t = static_cast<Resources *>(_o);
        switch (_id) {
        case 0: _t->RegisteredResourceEvent((*reinterpret_cast< const Event(*)>(_a[1]))); break;
        case 1: _t->ProcessedResourceEvents((*reinterpret_cast< const EventList(*)>(_a[1]))); break;
        case 2: _t->RegisteredResourceMimeType((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->RegisteredResourceTitle((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->RegisterResourceEvent((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< uint(*)>(_a[4])),(*reinterpret_cast< uint(*)>(_a[5]))); break;
        case 5: _t->RegisterResourceMimeType((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->RegisterResourceTitle((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Resources::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Resources::staticMetaObject = {
    { &Module::staticMetaObject, qt_meta_stringdata_Resources,
      qt_meta_data_Resources, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Resources::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Resources::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Resources::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Resources))
        return static_cast<void*>(const_cast< Resources*>(this));
    return Module::qt_metacast(_clname);
}

int Resources::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Module::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Resources::RegisteredResourceEvent(const Event & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Resources::ProcessedResourceEvents(const EventList & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Resources::RegisteredResourceMimeType(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Resources::RegisteredResourceTitle(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_END_MOC_NAMESPACE
