/*
 *   Copyright (C) 2012 Ivan Cukic <ivan.cukic(at)kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "Given.h"

namespace Jobs {
namespace Schedulers {

Given::Given(JobFactory * _condition, JobFactory * _then, JobFactory * _else, bool failOnElse, QObject * parent)
    : Abstract(parent), m_failOnElse(failOnElse)
{
    addJob(_condition);
    addJob(_then);
    addJob(_else);
}

Given::~Given()
{
}

void Given::jobFinished(int result)
{
    if (lastJobStarted() == 0) {
        int next = (result == 0) ? 1 : 2;

        if (hasJob(next)) {
            startJob(next);
            return;
        }

    }

    if (lastJobStarted() == 2 && m_failOnElse) {
        setError(1);

    } else {
        setError(result);

    }

    emitResult();
}



} // namespace Schedulers
} // namespace Jobs

