/****************************************************************************
** Meta object code from reading C++ file 'SlcPlugin.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/kactivities-4.13.3/src/service/plugins/slc/SlcPlugin.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SlcPlugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_SlcPlugin[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
      38,   10,

 // signals: signature, parameters, type, tag, flags
      74,   55,   54,   54, 0x05,

 // slots: signature, parameters, type, tag, flags
     118,  112,   54,   54, 0x08,
     162,  149,   54,   54, 0x08,
     216,  206,   54,   54, 0x08,
     265,   54,  257,   54, 0x0a,
     287,   54,  257,   54, 0x0a,
     314,   54,  257,   54, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_SlcPlugin[] = {
    "SlcPlugin\0org.kde.ActivityManager.SLC\0"
    "D-Bus Interface\0\0uri,mimetype,title\0"
    "focusChanged(QString,QString,QString)\0"
    "event\0registeredResourceEvent(Event)\0"
    "uri,mimetype\0registeredResourceMimeType(QString,QString)\0"
    "uri,title\0registeredResourceTitle(QString,QString)\0"
    "QString\0focussedResourceURI()\0"
    "focussedResourceMimetype()\0"
    "focussedResourceTitle()\0"
};

void SlcPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        SlcPlugin *_t = static_cast<SlcPlugin *>(_o);
        switch (_id) {
        case 0: _t->focusChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 1: _t->registeredResourceEvent((*reinterpret_cast< const Event(*)>(_a[1]))); break;
        case 2: _t->registeredResourceMimeType((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->registeredResourceTitle((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: { QString _r = _t->focussedResourceURI();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 5: { QString _r = _t->focussedResourceMimetype();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 6: { QString _r = _t->focussedResourceTitle();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData SlcPlugin::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject SlcPlugin::staticMetaObject = {
    { &Plugin::staticMetaObject, qt_meta_stringdata_SlcPlugin,
      qt_meta_data_SlcPlugin, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &SlcPlugin::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *SlcPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *SlcPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_SlcPlugin))
        return static_cast<void*>(const_cast< SlcPlugin*>(this));
    return Plugin::qt_metacast(_clname);
}

int SlcPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void SlcPlugin::focusChanged(const QString & _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
