/****************************************************************************
** Meta object code from reading C++ file 'Features.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kactivities-4.13.3/src/service/Features.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Features.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Features[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
       4,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
      42,    9,

 // slots: signature, parameters, type, tag, flags
      72,   64,   59,   58, 0x0a,
     102,   64,   59,   58, 0x0a,
     142,  128,   58,   58, 0x0a,
     193,  186,  174,   58, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_Features[] = {
    "Features\0org.kde.ActivityManager.Features\0"
    "D-Bus Interface\0\0bool\0feature\0"
    "IsFeatureOperational(QString)\0"
    "IsFeatureEnabled(QString)\0feature,value\0"
    "SetFeatureEnabled(QString,bool)\0"
    "QStringList\0module\0ListFeatures(QString)\0"
};

void Features::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        Features *_t = static_cast<Features *>(_o);
        switch (_id) {
        case 0: { bool _r = _t->IsFeatureOperational((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 1: { bool _r = _t->IsFeatureEnabled((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 2: _t->SetFeatureEnabled((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: { QStringList _r = _t->ListFeatures((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Features::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Features::staticMetaObject = {
    { &Module::staticMetaObject, qt_meta_stringdata_Features,
      qt_meta_data_Features, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Features::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Features::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Features::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Features))
        return static_cast<void*>(const_cast< Features*>(this));
    return Module::qt_metacast(_clname);
}

int Features::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Module::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
