/****************************************************************************
** Meta object code from reading C++ file 'Activities.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kactivities-4.13.3/src/service/Activities.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Activities.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Activities[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
      24,   16, // methods
       1,  136, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
      46,   11,

 // signals: signature, parameters, type, tag, flags
      72,   63,   62,   62, 0x05,
     104,   63,   62,   62, 0x05,
     127,   63,   62,   62, 0x05,
     152,   63,   62,   62, 0x05,
     177,   63,   62,   62, 0x05,
     216,  202,   62,   62, 0x05,
     267,  253,   62,   62, 0x05,
     304,   63,   62,   62, 0x05,
     344,  329,   62,   62, 0x05,

 // slots: signature, parameters, type, tag, flags
     386,   62,  378,   62, 0x0a,
     409,   63,  404,   62, 0x0a,
     442,  437,  378,   62, 0x0a,
     463,   63,   62,   62, 0x0a,
     486,   63,   62,   62, 0x0a,
     512,   63,  508,   62, 0x0a,
     535,   63,   62,   62, 0x0a,
     571,   62,  559,   62, 0x0a,
     594,  588,  559,   62, 0x0a,
     614,   63,  378,   62, 0x0a,
     636,  202,   62,   62, 0x0a,
     669,   63,  378,   62, 0x0a,
     691,  253,   62,   62, 0x0a,
     741,   62,  724,   62, 0x0a,
     786,   63,  773,   62, 0x0a,

 // properties: name, type, flags
     815,  378, 0x0a495003,

 // properties: notify_signal_id
       0,

       0        // eod
};

static const char qt_meta_stringdata_Activities[] = {
    "Activities\0org.kde.ActivityManager.Activities\0"
    "D-Bus Interface\0\0activity\0"
    "CurrentActivityChanged(QString)\0"
    "ActivityAdded(QString)\0ActivityStarted(QString)\0"
    "ActivityStopped(QString)\0"
    "ActivityRemoved(QString)\0activity,name\0"
    "ActivityNameChanged(QString,QString)\0"
    "activity,icon\0ActivityIconChanged(QString,QString)\0"
    "ActivityChanged(QString)\0activity,state\0"
    "ActivityStateChanged(QString,int)\0"
    "QString\0CurrentActivity()\0bool\0"
    "SetCurrentActivity(QString)\0name\0"
    "AddActivity(QString)\0StartActivity(QString)\0"
    "StopActivity(QString)\0int\0"
    "ActivityState(QString)\0RemoveActivity(QString)\0"
    "QStringList\0ListActivities()\0state\0"
    "ListActivities(int)\0ActivityName(QString)\0"
    "SetActivityName(QString,QString)\0"
    "ActivityIcon(QString)\0"
    "SetActivityIcon(QString,QString)\0"
    "ActivityInfoList\0ListActivitiesWithInformation()\0"
    "ActivityInfo\0ActivityInformation(QString)\0"
    "CurrentActivity\0"
};

void Activities::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        Activities *_t = static_cast<Activities *>(_o);
        switch (_id) {
        case 0: _t->CurrentActivityChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->ActivityAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->ActivityStarted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->ActivityStopped((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->ActivityRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->ActivityNameChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->ActivityIconChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->ActivityChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->ActivityStateChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: { QString _r = _t->CurrentActivity();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 10: { bool _r = _t->SetCurrentActivity((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 11: { QString _r = _t->AddActivity((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 12: _t->StartActivity((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->StopActivity((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: { int _r = _t->ActivityState((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        case 15: _t->RemoveActivity((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: { QStringList _r = _t->ListActivities();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 17: { QStringList _r = _t->ListActivities((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 18: { QString _r = _t->ActivityName((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 19: _t->SetActivityName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 20: { QString _r = _t->ActivityIcon((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 21: _t->SetActivityIcon((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 22: { ActivityInfoList _r = _t->ListActivitiesWithInformation();
            if (_a[0]) *reinterpret_cast< ActivityInfoList*>(_a[0]) = _r; }  break;
        case 23: { ActivityInfo _r = _t->ActivityInformation((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< ActivityInfo*>(_a[0]) = _r; }  break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Activities::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Activities::staticMetaObject = {
    { &Module::staticMetaObject, qt_meta_stringdata_Activities,
      qt_meta_data_Activities, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Activities::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Activities::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Activities::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Activities))
        return static_cast<void*>(const_cast< Activities*>(this));
    return Module::qt_metacast(_clname);
}

int Activities::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Module::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
#ifndef QT_NO_PROPERTIES
      else if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = CurrentActivity(); break;
        }
        _id -= 1;
    } else if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: SetCurrentActivity(*reinterpret_cast< QString*>(_v)); break;
        }
        _id -= 1;
    } else if (_c == QMetaObject::ResetProperty) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Activities::CurrentActivityChanged(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Activities::ActivityAdded(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Activities::ActivityStarted(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Activities::ActivityStopped(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Activities::ActivityRemoved(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Activities::ActivityNameChanged(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void Activities::ActivityIconChanged(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Activities::ActivityChanged(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Activities::ActivityStateChanged(const QString & _t1, int _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_END_MOC_NAMESPACE
