/****************************************************************************
** Meta object code from reading C++ file 'StatsPlugin.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.6)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/kactivities-4.13.3/src/service/plugins/sqlite/StatsPlugin.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StatsPlugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.6. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_StatsPlugin[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
      54,   12,

 // signals: signature, parameters, type, tag, flags
     102,   71,   70,   70, 0x05,
     175,  155,   70,   70, 0x05,
     231,  215,   70,   70, 0x05,

 // slots: signature, parameters, type, tag, flags
     264,  155,   70,   70, 0x0a,
     303,  215,   70,   70, 0x0a,
     342,  335,   70,   70, 0x08,
     363,   70,   70,   70, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_StatsPlugin[] = {
    "StatsPlugin\0org.kde.ActivityManager.Resources.Scoring\0"
    "D-Bus Interface\0\0activity,client,resource,score\0"
    "resourceScoreUpdated(QString,QString,QString,double)\0"
    "activity,count,what\0"
    "recentStatsDeleted(QString,int,QString)\0"
    "activity,months\0earlierStatsDeleted(QString,int)\0"
    "deleteRecentStats(QString,int,QString)\0"
    "deleteEarlierStats(QString,int)\0events\0"
    "addEvents(EventList)\0loadConfiguration()\0"
};

void StatsPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        StatsPlugin *_t = static_cast<StatsPlugin *>(_o);
        switch (_id) {
        case 0: _t->resourceScoreUpdated((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4]))); break;
        case 1: _t->recentStatsDeleted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 2: _t->earlierStatsDeleted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->deleteRecentStats((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 4: _t->deleteEarlierStats((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->addEvents((*reinterpret_cast< const EventList(*)>(_a[1]))); break;
        case 6: _t->loadConfiguration(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData StatsPlugin::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject StatsPlugin::staticMetaObject = {
    { &Plugin::staticMetaObject, qt_meta_stringdata_StatsPlugin,
      qt_meta_data_StatsPlugin, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &StatsPlugin::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *StatsPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *StatsPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_StatsPlugin))
        return static_cast<void*>(const_cast< StatsPlugin*>(this));
    return Plugin::qt_metacast(_clname);
}

int StatsPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Plugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void StatsPlugin::resourceScoreUpdated(const QString & _t1, const QString & _t2, const QString & _t3, double _t4)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void StatsPlugin::recentStatsDeleted(const QString & _t1, int _t2, const QString & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void StatsPlugin::earlierStatsDeleted(const QString & _t1, int _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_END_MOC_NAMESPACE
