/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a resourcesadaptor -i Resources.h -l Resources /usr/src/ports/kactivities/kactivities-4.13.3-2.x86_64/src/kactivities-4.13.3/src/common/dbus/org.kde.ActivityManager.Resources.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "resourcesadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ResourcesAdaptor
 */

ResourcesAdaptor::ResourcesAdaptor(Resources *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ResourcesAdaptor::~ResourcesAdaptor()
{
    // destructor
}

void ResourcesAdaptor::RegisterResourceEvent(const QString &application, uint windowId, const QString &uri, uint event, uint reason)
{
    // handle method call org.kde.ActivityManager.Resources.RegisterResourceEvent
    parent()->RegisterResourceEvent(application, windowId, uri, event, reason);
}

void ResourcesAdaptor::RegisterResourceMimeType(const QString &uri, const QString &mimetype)
{
    // handle method call org.kde.ActivityManager.Resources.RegisterResourceMimeType
    parent()->RegisterResourceMimeType(uri, mimetype);
}

void ResourcesAdaptor::RegisterResourceTitle(const QString &uri, const QString &title)
{
    // handle method call org.kde.ActivityManager.Resources.RegisterResourceTitle
    parent()->RegisterResourceTitle(uri, title);
}


#include "resourcesadaptor.moc"
