/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a slcadaptor -i SlcPlugin.h -l SlcPlugin /usr/src/ports/kactivities/kactivities-4.13.3-2.x86_64/src/kactivities-4.13.3/src/service/plugins/slc/org.kde.ActivityManager.SLC.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "slcadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class SLCAdaptor
 */

SLCAdaptor::SLCAdaptor(SlcPlugin *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

SLCAdaptor::~SLCAdaptor()
{
    // destructor
}

QString SLCAdaptor::focussedResourceMimetype()
{
    // handle method call org.kde.ActivityManager.SLC.focussedResourceMimetype
    return parent()->focussedResourceMimetype();
}

QString SLCAdaptor::focussedResourceTitle()
{
    // handle method call org.kde.ActivityManager.SLC.focussedResourceTitle
    return parent()->focussedResourceTitle();
}

QString SLCAdaptor::focussedResourceURI()
{
    // handle method call org.kde.ActivityManager.SLC.focussedResourceURI
    return parent()->focussedResourceURI();
}


#include "slcadaptor.moc"
