/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a featuresadaptor -i Features.h -l Features /usr/src/ports/kactivities/kactivities-4.13.3-2.x86_64/src/kactivities-4.13.3/src/common/dbus/org.kde.ActivityManager.Features.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "featuresadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class FeaturesAdaptor
 */

FeaturesAdaptor::FeaturesAdaptor(Features *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

FeaturesAdaptor::~FeaturesAdaptor()
{
    // destructor
}

bool FeaturesAdaptor::IsFeatureEnabled(const QString &feature)
{
    // handle method call org.kde.ActivityManager.Features.IsFeatureEnabled
    return parent()->IsFeatureEnabled(feature);
}

bool FeaturesAdaptor::IsFeatureOperational(const QString &feature)
{
    // handle method call org.kde.ActivityManager.Features.IsFeatureOperational
    return parent()->IsFeatureOperational(feature);
}

QStringList FeaturesAdaptor::ListFeatures(const QString &module)
{
    // handle method call org.kde.ActivityManager.Features.ListFeatures
    return parent()->ListFeatures(module);
}

void FeaturesAdaptor::SetFeatureEnabled(const QString &feature, bool value)
{
    // handle method call org.kde.ActivityManager.Features.SetFeatureEnabled
    parent()->SetFeatureEnabled(feature, value);
}


#include "featuresadaptor.moc"
