/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a scoringadaptor -i StatsPlugin.h -l StatsPlugin /usr/src/ports/kactivities/kactivities-4.13.3-2.x86_64/src/kactivities-4.13.3/src/service/plugins/sqlite/org.kde.ActivityManager.Resources.Scoring.xml
 *
 * qdbusxml2cpp is Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "scoringadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ScoringAdaptor
 */

ScoringAdaptor::ScoringAdaptor(StatsPlugin *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ScoringAdaptor::~ScoringAdaptor()
{
    // destructor
}

void ScoringAdaptor::deleteEarlierStats(const QString &activity, int months)
{
    // handle method call org.kde.ActivityManager.Resources.Scoring.deleteEarlierStats
    parent()->deleteEarlierStats(activity, months);
}

void ScoringAdaptor::deleteRecentStats(const QString &activity, int count, const QString &what)
{
    // handle method call org.kde.ActivityManager.Resources.Scoring.deleteRecentStats
    parent()->deleteRecentStats(activity, count, what);
}


#include "scoringadaptor.moc"
